import React, {Fragment, Component} from 'react';
import PropTypes from 'prop-types';
import Select, {components} from 'react-select';
import {
    FormGroup,
    Label,
    Row,
    Col,
    UncontrolledTooltip,
    Button,
} from 'reactstrap';
import Loading from '../Loading';
import {TextField} from './index';
import {
    noPadding,
    styleInvalid,
    styleValid,
    small,
    flexR,
    toolTip,
    padding,
    toolTipIcon,
    smallAlt,
    list,
    buttonWarning,
    listItem
} from '../common/styles/index';
import {find, map, remove, size} from "lodash";
import OptionsService from "../services/network/fetchSelectOptions";

class SelectMultiple extends Component {
    static propTypes = {
        label: PropTypes.string.isRequired,
        control: PropTypes.string.isRequired,
        options: PropTypes.arrayOf(PropTypes.shape({})).isRequired,
        onChange: PropTypes.func.isRequired,
        onChangeOthers: PropTypes.func.isRequired,
        valueOthers: PropTypes.string,
        value: PropTypes.oneOfType([
            PropTypes.string,
            PropTypes.number
        ]).isRequired,
        onInputChange: PropTypes.func,
        getValue: PropTypes.func,
        defaultOption: PropTypes.bool,
        placeholder: PropTypes.string,
        disabled: PropTypes.bool,
        filterOption: PropTypes.oneOfType([
            PropTypes.any
        ]),
        helpText: PropTypes.bool,
        error: PropTypes.shape.isRequired,
        searchAmount: PropTypes.number,
        token: PropTypes.string,
        source: PropTypes.arrayOf(PropTypes.shape({})),
    };

    static defaultProps = {
        source: [],
        searchAmount: 3,
        options: [],
        defaultOption: false,
        placeholder: '[Seleccione]',
        valueOthers: '',
        onInputChange: null,
        disabled: false,
        filterOption: null,
        helpText: null,
        token: 'An invalid Token',
    };
    
    constructor(props) {
        super(props);
        this.state = {
            options: this.props.options ? this.props.options : [],
            isSearching: false,
            result: ''
        }
    }

    handleInPutChange(e) {
            this.timeout = setTimeout(() => {
                if (e !== undefined && this.props.source !== [] && e.length > this.props.searchAmount && !this.state.isSearching) {
                    this.setState({isSearching: true})
                    OptionsService.fetchOptions(this.props.token, e, this.props.source.query).
                    then(result => {
                        if (result.message){
                            this.setState({
                                options:( [{label: '[Opcion no encontrada]', value: null}]),
                                isSearching: false,
                                result: result.message
                            })
                        } else {
                                this.setState({
                                    isSearching: false,
                                    options: this.props.source.reducer(result),
                                    result: ''
                                })}})}}, 500)}

    handleGetValue(e) {
        const value = {label: e.label, value: e.value}
        if (this.props.value.length === 0 || this.props.value.value === null) {
            this.props.onChange([value], this.props.control)
        } else {
            if (!find(this.props.value, (value) => {
                return (value.value === e.value)
            })) {
                this.setState({result: ''})
                this.props.onChange(this.props.value.concat([value]), this.props.control)
            } else {
                this.setState({result: 'No se pueden ingresar valores duplicados'})
            }
        }
    }

    handleDelete(e) {
        const result = remove(this.props.value, (value) => {
            return value.value !== e.target.value
        })
        if (size(result) === 0) {
            this.props.onChange([], this.props.control)
        } else {
            this.props.onChange(result, this.props.control)
        }
    }

        render()
        {
            const customComponents = {
                NoOptionsMessage: props => (
                    <components.NoOptionsMessage {...props}>
                        No se encontraron resultados&nbsp;
                    </components.NoOptionsMessage>
                )
            };
            const {value} = this.props
            return (
                <FormGroup>
                    <Row style={noPadding}>
                        <Col style={noPadding}>
                            <Fragment>
                                <Label
                                    style={{...small, ...noPadding, ...flexR,...padding}}>
                                    <p
                                        style={noPadding}>
                                        {this.props.label+" "}
                                    </p>
                                    {this.props.helpText && (
                                        <p style={{...noPadding, ...toolTipIcon, ...smallAlt}}  id={'ref'+this.props.control}>
                                            {String.fromCharCode(0x24D8)}
                                        </p>
                                    )}
                                </Label>
                                <Select
                                    id={this.props.control}
                                    value={null}
                                    defaultValue={this.props.defaultOption ? {value: '', label: '[Todos]'} : false}
                                    onChange={e=>this.handleGetValue(e)}
                                    onInputChange={e=>this.handleInPutChange(e)}
                                    options={this.state.options}
                                    components={customComponents}
                                    filterOption={this.props.filterOption}
                                    isSearchable
                                    isDisabled={this.props.disabled}
                                    placeholder={this.props.placeholder}
                                    clearValueText="Limpiar Filtro"
                                    isLoading={this.state.isSearching}
                                    closeMenuOnSelect
                                />
                                <ul style={{...list}}>
                                    <div>
                                        {(value.value !== null) && (
                                            map(value, ((value, key) => (
                                                    <div key={key}
                                                         style={{...listItem, ...{backgroundColor: (key % 2 === 0 ? 'lightgray' : 'white')}}}>
                                                        <Label>{value.label}</Label>
                                                        <Button stlye={{...buttonWarning}}
                                                                color="danger" size="sm"
                                                                value={value.value}
                                                                onClick={e => this.handleDelete(e)}>x</Button>
                                                    </div>
                                                )
                                            )))
                                        }
                                    </div>
                                </ul>
                                {this.props.helpText && (
                                    <UncontrolledTooltip
                                        placement="top"
                                        target={'ref' + this.props.control}
                                        style={toolTip}>
                                        {this.props.helpText}
                                    </UncontrolledTooltip>
                                )}
                                {this.state.isSearching && (
                                    <Loading
                                        label="Estamos buscando tu consulta"
                                    />
                                )}
                                {
                                    (this.props.error.message !== '')  &&
                                    (
                                        <p style={this.props.error.status === 'valid' ? styleValid : styleInvalid}>
                                            {this.props.error.message + (this.state.result !== '' ?(' -> '+this.state.result) : '' )}
                                    </p>)
                                }
                            </Fragment>
                        </Col>
                        {
                            this.props.supportsWriteOthers && (
                                <div>
                                    &nbsp;&nbsp;
                                </div>
                            )
                        }
                        {this.props.supportsWriteOthers  && (
                            <Col style={noPadding}>
                                <TextField
                                    name={`${this.props.control}Others`}
                                    label={`Otro ${this.props.label}`}
                                    value={this.props.valueOthers}
                                    helpText={null}
                                    disabled={value.value !== 'idAnother'}
                                    onChange={e => this.props.onChangeOthers(e)}
                                    error={this.props.error}
                                    type={'textField'}/>
                            </Col>
                        )}
                    </Row>
                </FormGroup>
            )
        }
    }


export default SelectMultiple
